@@ -1,16 +1,20 @@
 import contextlib
+import inspect
+import logging
 import pprint
-import sys
 import traceback
 import types
-import typing
+from collections.abc import Callable, Sequence
 from dataclasses import dataclass
+from typing import Any, Optional
+
+import sys
 
-from mitmproxy import controller
-from mitmproxy import hooks
 from mitmproxy import exceptions
 from mitmproxy import flow
-from . import ctx
+from mitmproxy import hooks
+
+logger = logging.getLogger(__name__)
 
 
 def _get_name(itm):
@@ -45,59 +49,52 @@ def safecall():
         raise
     except Exception:
         etype, value, tb = sys.exc_info()
+        tb = cut_traceback(tb, "invoke_addon_sync")
         tb = cut_traceback(tb, "invoke_addon")
-        ctx.log.error(
-            "Addon error: %s" % "".join(
-                traceback.format_exception(etype, value, tb)
-            )
+        logger.error(
+            "Addon error: %s" % "".join(traceback.format_exception(etype, value, tb))
         )
 
 
 class Loader:
     """
-        A loader object is passed to the load() event when addons start up.
+    A loader object is passed to the load() event when addons start up.
     """
 
     def __init__(self, master):
         self.master = master
 
     def add_option(
-            self,
-            name: str,
-            typespec: type,
-            default: typing.Any,
-            help: str,
-            choices: typing.Optional[typing.Sequence[str]] = None
+        self,
+        name: str,
+        typespec: type,
+        default: Any,
+        help: str,
+        choices: Optional[Sequence[str]] = None,
     ) -> None:
         """
-            Add an option to mitmproxy.
+        Add an option to mitmproxy.
 
-            Help should be a single paragraph with no linebreaks - it will be
-            reflowed by tools. Information on the data type should be omitted -
-            it will be generated and added by tools as needed.
+        Help should be a single paragraph with no linebreaks - it will be
+        reflowed by tools. Information on the data type should be omitted -
+        it will be generated and added by tools as needed.
         """
         if name in self.master.options:
             existing = self.master.options._options[name]
             same_signature = (
-                    existing.name == name and
-                    existing.typespec == typespec and
-                    existing.default == default and
-                    existing.help == help and
-                    existing.choices == choices
+                existing.name == name
+                and existing.typespec == typespec
+                and existing.default == default
+                and existing.help == help
+                and existing.choices == choices
             )
             if same_signature:
                 return
             else:
-                ctx.log.warn("Over-riding existing option %s" % name)
-        self.master.options.add_option(
-            name,
-            typespec,
-            default,
-            help,
-            choices
-        )
+                logger.warning("Over-riding existing option %s" % name)
+        self.master.options.add_option(name, typespec, default, help, choices)
 
-    def add_command(self, path: str, func: typing.Callable) -> None:
+    def add_command(self, path: str, func: Callable) -> None:
         """Add a command to mitmproxy.
 
         Unless you are generating commands programatically,
@@ -108,7 +105,7 @@ class Loader:
 
 def traverse(chain):
     """
-        Recursively traverse an addon chain.
+    Recursively traverse an addon chain.
     """
     for a in chain:
         yield a
@@ -123,6 +120,7 @@ class LoadHook(hooks.Hook):
     object, which contains methods for adding options and commands. This
     method is where the addon configures itself.
     """
+
     loader: Loader
 
 
@@ -133,55 +131,58 @@ class AddonManager:
         self.master = master
         master.options.changed.connect(self._configure_all)
 
-    def _configure_all(self, options, updated):
+    def _configure_all(self, updated):
         self.trigger(hooks.ConfigureHook(updated))
 
     def clear(self):
         """
-            Remove all addons.
+        Remove all addons.
         """
         for a in self.chain:
-            self.invoke_addon(a, hooks.DoneHook())
+            self.invoke_addon_sync(a, hooks.DoneHook())
         self.lookup = {}
         self.chain = []
 
     def get(self, name):
         """
-            Retrieve an addon by name. Addon names are equal to the .name
-            attribute on the instance, or the lower case class name if that
-            does not exist.
+        Retrieve an addon by name. Addon names are equal to the .name
+        attribute on the instance, or the lower case class name if that
+        does not exist.
         """
         return self.lookup.get(name, None)
 
     def register(self, addon):
         """
-            Register an addon, call its load event, and then register all its
-            sub-addons. This should be used by addons that dynamically manage
-            addons.
+        Register an addon, call its load event, and then register all its
+        sub-addons. This should be used by addons that dynamically manage
+        addons.
 
-            If the calling addon is already running, it should follow with
-            running and configure events. Must be called within a current
-            context.
+        If the calling addon is already running, it should follow with
+        running and configure events. Must be called within a current
+        context.
         """
         api_changes = {
             # mitmproxy 6 -> mitmproxy 7
-            "clientconnect": "client_connected",
-            "clientdisconnect": "client_disconnected",
-            "serverconnect": "server_connect and server_connected",
-            "serverdisconnect": "server_disconnected",
+            "clientconnect": f"The clientconnect event has been removed, use client_connected instead",
+            "clientdisconnect": f"The clientdisconnect event has been removed, use client_disconnected instead",
+            "serverconnect": "The serverconnect event has been removed, use server_connect and server_connected instead",
+            "serverdisconnect": f"The serverdisconnect event has been removed, use server_disconnected instead",
+            # mitmproxy 8 -> mitmproxy 9
+            "add_log": "The add_log event has been deprecated, use Python's builtin logging module instead",
         }
         for a in traverse([addon]):
-            for old, new in api_changes.items():
+            for old, msg in api_changes.items():
                 if hasattr(a, old):
-                    ctx.log.warn(f"The {old} event has been removed, use {new} instead. "
-                                 f"For more details, see https://docs.mitmproxy.org/stable/addons-events/.")
+                    logger.warning(
+                        f"{msg}. For more details, see https://docs.mitmproxy.org/dev/addons-api-changelog/."
+                    )
             name = _get_name(a)
             if name in self.lookup:
                 raise exceptions.AddonManagerError(
                     "An addon called '%s' already exists." % name
                 )
         l = Loader(self.master)
-        self.invoke_addon(addon, LoadHook(l))
+        self.invoke_addon_sync(addon, LoadHook(l))
         for a in traverse([addon]):
             name = _get_name(a)
             self.lookup[name] = a
@@ -192,19 +193,19 @@ class AddonManager:
 
     def add(self, *addons):
         """
-            Add addons to the end of the chain, and run their load event.
-            If any addon has sub-addons, they are registered.
+        Add addons to the end of the chain, and run their load event.
+        If any addon has sub-addons, they are registered.
         """
         for i in addons:
             self.chain.append(self.register(i))
 
     def remove(self, addon):
         """
-            Remove an addon and all its sub-addons.
+        Remove an addon and all its sub-addons.
 
-            If the addon is not in the chain - that is, if it's managed by a
-            parent addon - it's the parent's responsibility to remove it from
-            its own addons attribute.
+        If the addon is not in the chain - that is, if it's managed by a
+        parent addon - it's the parent's responsibility to remove it from
+        its own addons attribute.
         """
         for a in traverse([addon]):
             n = _get_name(a)
@@ -212,7 +213,7 @@ class AddonManager:
                 raise exceptions.AddonManagerError("No such addon: %s" % n)
             self.chain = [i for i in self.chain if i is not a]
             del self.lookup[_get_name(a)]
-        self.invoke_addon(addon, hooks.DoneHook())
+        self.invoke_addon_sync(addon, hooks.DoneHook())
 
     def __len__(self):
         return len(self.chain)
@@ -226,42 +227,25 @@ class AddonManager:
 
     async def handle_lifecycle(self, event: hooks.Hook):
         """
-            Handle a lifecycle event.
+        Handle a lifecycle event.
         """
         message = event.args()[0]
-        if not hasattr(message, "reply"):  # pragma: no cover
-            raise exceptions.ControlException(
-                "Message %s has no reply attribute" % message
-            )
 
-        # We can use DummyReply objects multiple times. We only clear them up on
-        # the next handler so that we can access value and state in the
-        # meantime.
-        if isinstance(message.reply, controller.DummyReply):
-            message.reply.reset()
-
-        self.trigger(event)
-
-        if message.reply.state == "start":
-            message.reply.take()
-            message.reply.commit()
-
-            if isinstance(message.reply, controller.DummyReply):
-                message.reply.mark_reset()
+        await self.trigger_event(event)
 
         if isinstance(message, flow.Flow):
-            self.trigger(hooks.UpdateHook([message]))
+            await self.trigger_event(hooks.UpdateHook([message]))
 
-    def invoke_addon(self, addon, event: hooks.Hook):
+    def _iter_hooks(self, addon, event: hooks.Hook):
         """
-            Invoke an event on an addon and all its children.
+        Enumerate all hook callables belonging to the given addon
         """
         assert isinstance(event, hooks.Hook)
         for a in traverse([addon]):
             func = getattr(a, event.name, None)
             if func:
                 if callable(func):
-                    func(*event.args())
+                    yield a, func
                 elif isinstance(func, types.ModuleType):
                     # we gracefully exclude module imports with the same name as hooks.
                     # For example, a user may have "from mitmproxy import log" in an addon,
@@ -273,13 +257,48 @@ class AddonManager:
                         f"Addon handler {event.name} ({a}) not callable"
                     )
 
-    def trigger(self, event: hooks.Hook):
+    async def invoke_addon(self, addon, event: hooks.Hook):
         """
-            Trigger an event across all addons.
+        Asynchronously invoke an event on an addon and all its children.
+        """
+        for addon, func in self._iter_hooks(addon, event):
+            res = func(*event.args())
+            # Support both async and sync hook functions
+            if res is not None and inspect.isawaitable(res):
+                await res
+
+    def invoke_addon_sync(self, addon, event: hooks.Hook):
+        """
+        Invoke an event on an addon and all its children.
+        """
+        for addon, func in self._iter_hooks(addon, event):
+            if inspect.iscoroutinefunction(func):
+                raise exceptions.AddonManagerError(
+                    f"Async handler {event.name} ({addon}) cannot be called from sync context"
+                )
+            func(*event.args())
+
+    async def trigger_event(self, event: hooks.Hook):
+        """
+        Asynchronously trigger an event across all addons.
         """
         for i in self.chain:
             try:
                 with safecall():
-                    self.invoke_addon(i, event)
+                    await self.invoke_addon(i, event)
+            except exceptions.AddonHalt:
+                return
+
+    def trigger(self, event: hooks.Hook):
+        """
+        Trigger an event across all addons.
+
+        This API is discouraged and may be deprecated in the future.
+        Use `trigger_event()` instead, which provides the same functionality but supports async hooks.
+        """
+        for i in self.chain:
+            try:
+                with safecall():
+                    self.invoke_addon_sync(i, event)
             except exceptions.AddonHalt:
                 return
