@@ -1,8 +1,10 @@
+import logging
 import mimetypes
 import re
-import typing
 import urllib.parse
+from collections.abc import Sequence
 from pathlib import Path
+from typing import NamedTuple
 
 from werkzeug.security import safe_join
 
@@ -10,7 +12,7 @@ from mitmproxy import ctx, exceptions, flowfilter, http, version
 from mitmproxy.utils.spec import parse_spec
 
 
-class MapLocalSpec(typing.NamedTuple):
+class MapLocalSpec(NamedTuple):
     matches: flowfilter.TFilter
     regex: str
     local_path: Path
@@ -38,16 +40,13 @@ def _safe_path_join(root: Path, untrusted: str) -> Path:
     This is a convenience wrapper for werkzeug's safe_join,
     raising a ValueError if the path is malformed."""
     untrusted_parts = Path(untrusted).parts
-    joined = safe_join(
-        root.as_posix(),
-        *untrusted_parts
-    )
+    joined = safe_join(root.as_posix(), *untrusted_parts)
     if joined is None:
         raise ValueError("Untrusted paths.")
     return Path(joined)
 
 
-def file_candidates(url: str, spec: MapLocalSpec) -> typing.List[Path]:
+def file_candidates(url: str, spec: MapLocalSpec) -> list[Path]:
     """
     Get all potential file candidates given a URL and a mapping spec ordered by preference.
     This function already assumes that the spec regex matches the URL.
@@ -69,10 +68,7 @@ def file_candidates(url: str, spec: MapLocalSpec) -> typing.List[Path]:
         if decoded_suffix != escaped_suffix:
             suffix_candidates.extend([escaped_suffix, f"{escaped_suffix}/index.html"])
         try:
-            return [
-                _safe_path_join(spec.local_path, x)
-                for x in suffix_candidates
-            ]
+            return [_safe_path_join(spec.local_path, x) for x in suffix_candidates]
         except ValueError:
             return []
     else:
@@ -81,16 +77,18 @@ def file_candidates(url: str, spec: MapLocalSpec) -> typing.List[Path]:
 
 class MapLocal:
     def __init__(self):
-        self.replacements: typing.List[MapLocalSpec] = []
+        self.replacements: list[MapLocalSpec] = []
 
     def load(self, loader):
         loader.add_option(
-            "map_local", typing.Sequence[str], [],
+            "map_local",
+            Sequence[str],
+            [],
             """
             Map remote resources to a local file using a pattern of the form
             "[/flow-filter]/url-regex/file-or-directory-path", where the
             separator can be any character.
-            """
+            """,
         )
 
     def configure(self, updated):
@@ -100,12 +98,14 @@ class MapLocal:
                 try:
                     spec = parse_map_local_spec(option)
                 except ValueError as e:
-                    raise exceptions.OptionsError(f"Cannot parse map_local option {option}: {e}") from e
+                    raise exceptions.OptionsError(
+                        f"Cannot parse map_local option {option}: {e}"
+                    ) from e
 
                 self.replacements.append(spec)
 
     def request(self, flow: http.HTTPFlow) -> None:
-        if flow.response or flow.error or (flow.reply and flow.reply.state == "taken"):
+        if flow.response or flow.error or not flow.live:
             return
 
         url = flow.request.pretty_url
@@ -126,9 +126,7 @@ class MapLocal:
                         break
 
                 if local_file:
-                    headers = {
-                        "Server": version.MITMPROXY
-                    }
+                    headers = {"Server": version.MITMPROXY}
                     mimetype = mimetypes.guess_type(str(local_file))[0]
                     if mimetype:
                         headers["Content-Type"] = mimetype
@@ -136,16 +134,14 @@ class MapLocal:
                     try:
                         contents = local_file.read_bytes()
                     except OSError as e:
-                        ctx.log.warn(f"Could not read file: {e}")
+                        logging.warning(f"Could not read file: {e}")
                         continue
 
-                    flow.response = http.Response.make(
-                        200,
-                        contents,
-                        headers
-                    )
+                    flow.response = http.Response.make(200, contents, headers)
                     # only set flow.response once, for the first matching rule
                     return
         if all_candidates:
             flow.response = http.Response.make(404)
-            ctx.log.info(f"None of the local file candidates exist: {', '.join(str(x) for x in all_candidates)}")
+            logging.info(
+                f"None of the local file candidates exist: {', '.join(str(x) for x in all_candidates)}"
+            )
