@@ -1,7 +1,8 @@
 import asyncio
+import logging
 import os.path
 import sys
-import typing
+from typing import BinaryIO, Optional
 
 from mitmproxy import ctx
 from mitmproxy import exceptions
@@ -12,34 +13,30 @@ from mitmproxy import command
 
 class ReadFile:
     """
-        An addon that handles reading from file on startup.
+    An addon that handles reading from file on startup.
     """
+
     def __init__(self):
         self.filter = None
         self.is_reading = False
 
     def load(self, loader):
+        loader.add_option("rfile", Optional[str], None, "Read flows from file.")
         loader.add_option(
-            "rfile", typing.Optional[str], None,
-            "Read flows from file."
-        )
-        loader.add_option(
-            "readfile_filter", typing.Optional[str], None,
-            "Read only matching flows."
+            "readfile_filter", Optional[str], None, "Read only matching flows."
         )
 
     def configure(self, updated):
         if "readfile_filter" in updated:
-            filt = None
             if ctx.options.readfile_filter:
-                filt = flowfilter.parse(ctx.options.readfile_filter)
-                if not filt:
-                    raise exceptions.OptionsError(
-                        "Invalid readfile filter: %s" % ctx.options.readfile_filter
-                    )
-            self.filter = filt
+                try:
+                    self.filter = flowfilter.parse(ctx.options.readfile_filter)
+                except ValueError as e:
+                    raise exceptions.OptionsError(str(e)) from e
+            else:
+                self.filter = None
 
-    async def load_flows(self, fo: typing.IO[bytes]) -> int:
+    async def load_flows(self, fo: BinaryIO) -> int:
         cnt = 0
         freader = io.FlowReader(fo)
         try:
@@ -48,11 +45,11 @@ class ReadFile:
                     continue
                 await ctx.master.load_flow(flow)
                 cnt += 1
-        except (IOError, exceptions.FlowReadException) as e:
+        except (OSError, exceptions.FlowReadException) as e:
             if cnt:
-                ctx.log.warn("Flow file corrupted - loaded %i flows." % cnt)
+                logging.warning("Flow file corrupted - loaded %i flows." % cnt)
             else:
-                ctx.log.error("Flow file corrupted.")
+                logging.error("Flow file corrupted.")
             raise exceptions.FlowReadException(str(e)) from e
         else:
             return cnt
@@ -62,8 +59,8 @@ class ReadFile:
         try:
             with open(path, "rb") as f:
                 return await self.load_flows(f)
-        except IOError as e:
-            ctx.log.error("Cannot load flows: {}".format(e))
+        except OSError as e:
+            logging.error(f"Cannot load flows: {e}")
             raise exceptions.FlowReadException(str(e)) from e
 
     async def doread(self, rfile):
@@ -77,7 +74,7 @@ class ReadFile:
 
     def running(self):
         if ctx.options.rfile:
-            asyncio.get_event_loop().create_task(self.doread(ctx.options.rfile))
+            asyncio.get_running_loop().create_task(self.doread(ctx.options.rfile))
 
     @command.command("readfile.reading")
     def reading(self) -> bool:
@@ -86,6 +83,7 @@ class ReadFile:
 
 class ReadFileStdin(ReadFile):
     """Support the special case of "-" for reading from stdin"""
+
     async def load_flows_from_path(self, path: str) -> int:
         if path == "-":  # pragma: no cover
             # Need to think about how to test this. This function is scheduled
