@@ -1,6 +1,8 @@
 import hashlib
-import typing
+import logging
 import urllib
+from collections.abc import Hashable, Sequence
+from typing import Any, Optional
 
 import mitmproxy.types
 from mitmproxy import command, hooks
@@ -11,7 +13,7 @@ from mitmproxy import io
 
 
 class ServerPlayback:
-    flowmap: typing.Dict[typing.Hashable, typing.List[http.HTTPFlow]]
+    flowmap: dict[Hashable, list[http.HTTPFlow]]
     configured: bool
 
     def __init__(self):
@@ -20,69 +22,92 @@ class ServerPlayback:
 
     def load(self, loader):
         loader.add_option(
-            "server_replay_kill_extra", bool, False,
-            "Kill extra requests during replay."
+            "server_replay_kill_extra",
+            bool,
+            False,
+            "Kill extra requests during replay (for which no replayable response was found).",
         )
         loader.add_option(
-            "server_replay_nopop", bool, False,
+            "server_replay_nopop",
+            bool,
+            False,
             """
             Don't remove flows from server replay state after use. This makes it
             possible to replay same response multiple times.
-            """
+            """,
         )
         loader.add_option(
-            "server_replay_refresh", bool, True,
+            "server_replay_refresh",
+            bool,
+            True,
             """
             Refresh server replay responses by adjusting date, expires and
             last-modified headers, as well as adjusting cookie expiration.
+            """,
+        )
+        loader.add_option(
+            "server_replay_use_headers",
+            Sequence[str],
+            [],
             """
-        )
-        loader.add_option(
-            "server_replay_use_headers", typing.Sequence[str], [],
-            "Request headers to be considered during replay."
-        )
-        loader.add_option(
-            "server_replay", typing.Sequence[str], [],
-            "Replay server responses from a saved file."
-        )
-        loader.add_option(
-            "server_replay_ignore_content", bool, False,
-            "Ignore request's content while searching for a saved flow to replay."
-        )
-        loader.add_option(
-            "server_replay_ignore_params", typing.Sequence[str], [],
-            """
-            Request's parameters to be ignored while searching for a saved flow
+            Request headers that need to match while searching for a saved flow
             to replay.
-            """
+            """,
         )
         loader.add_option(
-            "server_replay_ignore_payload_params", typing.Sequence[str], [],
+            "server_replay",
+            Sequence[str],
+            [],
+            "Replay server responses from a saved file.",
+        )
+        loader.add_option(
+            "server_replay_ignore_content",
+            bool,
+            False,
+            "Ignore request content while searching for a saved flow to replay.",
+        )
+        loader.add_option(
+            "server_replay_ignore_params",
+            Sequence[str],
+            [],
             """
-            Request's payload parameters (application/x-www-form-urlencoded or
+            Request parameters to be ignored while searching for a saved flow
+            to replay.
+            """,
+        )
+        loader.add_option(
+            "server_replay_ignore_payload_params",
+            Sequence[str],
+            [],
+            """
+            Request payload parameters (application/x-www-form-urlencoded or
             multipart/form-data) to be ignored while searching for a saved flow
             to replay.
-            """
+            """,
         )
         loader.add_option(
-            "server_replay_ignore_host", bool, False,
+            "server_replay_ignore_host",
+            bool,
+            False,
             """
-            Ignore request's destination host while searching for a saved flow
+            Ignore request destination host while searching for a saved flow
             to replay.
-            """
+            """,
         )
         loader.add_option(
-            "server_replay_ignore_port", bool, False,
+            "server_replay_ignore_port",
+            bool,
+            False,
             """
-            Ignore request's destination port while searching for a saved flow
+            Ignore request destination port while searching for a saved flow
             to replay.
-            """
+            """,
         )
 
     @command.command("replay.server")
-    def load_flows(self, flows: typing.Sequence[flow.Flow]) -> None:
+    def load_flows(self, flows: Sequence[flow.Flow]) -> None:
         """
-            Replay server responses from flows.
+        Replay server responses from flows.
         """
         self.flowmap = {}
         for f in flows:
@@ -102,30 +127,31 @@ class ServerPlayback:
     @command.command("replay.server.stop")
     def clear(self) -> None:
         """
-            Stop server replay.
+        Stop server replay.
         """
         self.flowmap = {}
         ctx.master.addons.trigger(hooks.UpdateHook([]))
 
     @command.command("replay.server.count")
     def count(self) -> int:
-        return sum([len(i) for i in self.flowmap.values()])
+        return sum(len(i) for i in self.flowmap.values())
 
-    def _hash(self, flow: http.HTTPFlow) -> typing.Hashable:
+    def _hash(self, flow: http.HTTPFlow) -> Hashable:
         """
-            Calculates a loose hash of the flow request.
+        Calculates a loose hash of the flow request.
         """
         r = flow.request
         _, _, path, _, query, _ = urllib.parse.urlparse(r.url)
         queriesArray = urllib.parse.parse_qsl(query, keep_blank_values=True)
 
-        key: typing.List[typing.Any] = [str(r.scheme), str(r.method), str(path)]
+        key: list[Any] = [str(r.scheme), str(r.method), str(path)]
         if not ctx.options.server_replay_ignore_content:
             if ctx.options.server_replay_ignore_payload_params and r.multipart_form:
                 key.extend(
                     (k, v)
                     for k, v in r.multipart_form.items(multi=True)
-                    if k.decode(errors="replace") not in ctx.options.server_replay_ignore_payload_params
+                    if k.decode(errors="replace")
+                    not in ctx.options.server_replay_ignore_payload_params
                 )
             elif ctx.options.server_replay_ignore_payload_params and r.urlencoded_form:
                 key.extend(
@@ -156,23 +182,19 @@ class ServerPlayback:
                 v = r.headers.get(i)
                 headers.append((i, v))
             key.append(headers)
-        return hashlib.sha256(
-            repr(key).encode("utf8", "surrogateescape")
-        ).digest()
+        return hashlib.sha256(repr(key).encode("utf8", "surrogateescape")).digest()
 
-    def next_flow(self, flow: http.HTTPFlow) -> typing.Optional[http.HTTPFlow]:
+    def next_flow(self, flow: http.HTTPFlow) -> Optional[http.HTTPFlow]:
         """
-            Returns the next flow object, or None if no matching flow was
-            found.
+        Returns the next flow object, or None if no matching flow was
+        found.
         """
         hash = self._hash(flow)
         if hash in self.flowmap:
             if ctx.options.server_replay_nopop:
-                return next((
-                    flow
-                    for flow in self.flowmap[hash]
-                    if flow.response
-                ), None)
+                return next(
+                    (flow for flow in self.flowmap[hash] if flow.response), None
+                )
             else:
                 ret = self.flowmap[hash].pop(0)
                 while not ret.response:
@@ -207,7 +229,7 @@ class ServerPlayback:
                 f.response = response
                 f.is_replay = "response"
             elif ctx.options.server_replay_kill_extra:
-                ctx.log.warn(
+                logging.warning(
                     "server_playback: killed non-replay request {}".format(
                         f.request.url
                     )
