@@ -5,6 +5,7 @@ from typing import Optional
 from mitmproxy import exceptions
 from mitmproxy import ctx
 from mitmproxy import http
+from mitmproxy.proxy import mode_specs
 from mitmproxy.utils import strutils
 
 
@@ -52,7 +53,7 @@ class UpstreamAuth:
 
     def requestheaders(self, f: http.HTTPFlow):
         if self.auth:
-            if ctx.options.mode.startswith("upstream") and f.request.scheme == "http":
+            if isinstance(f.client_conn.proxy_mode, mode_specs.UpstreamMode) and f.request.scheme == "http":
                 f.request.headers["Proxy-Authorization"] = self.auth
-            elif ctx.options.mode.startswith("reverse"):
+            elif isinstance(f.client_conn.proxy_mode, mode_specs.ReverseMode):
                 f.request.headers["Authorization"] = self.auth
