@@ -15,5 +15,7 @@ class ViewURLEncoded(base.View):
         d = url.decode(data)
         return "URLEncoded form", base.format_pairs(d)
 
-    def render_priority(self, data: bytes, *, content_type: Optional[str] = None, **metadata) -> float:
+    def render_priority(
+        self, data: bytes, *, content_type: Optional[str] = None, **metadata
+    ) -> float:
         return float(bool(data) and content_type == "application/x-www-form-urlencoded")
