@@ -1,5 +1,6 @@
 import collections
-from typing import NamedTuple
+import logging
+from typing import Dict, List, NamedTuple, Tuple
 
 import h2.config
 import h2.connection
@@ -9,16 +10,29 @@ import h2.settings
 import h2.stream
 
 
+logger = logging.getLogger(__name__)
+
+
 class H2ConnectionLogger(h2.config.DummyLogger):
-    def __init__(self, name: str):
+    def __init__(self, peername: tuple, conn_type: str):
         super().__init__()
-        self.name = name
+        self.peername = peername
+        self.conn_type = conn_type
 
     def debug(self, fmtstr, *args):
-        print(f"{self.name} h2 (debug): {fmtstr % args}")
+        logger.debug(
+            f"{self.conn_type} {fmtstr}",
+            *args,
+            extra={"client": self.peername}
+        )
 
     def trace(self, fmtstr, *args):
-        print(f"{self.name} h2 (trace): {fmtstr % args}")
+        logger.log(
+            logging.DEBUG - 1,
+            f"{self.conn_type} {fmtstr}",
+            *args,
+            extra={"client": self.peername}
+        )
 
 
 class SendH2Data(NamedTuple):
@@ -34,10 +48,12 @@ class BufferedH2Connection(h2.connection.H2Connection):
     """
 
     stream_buffers: collections.defaultdict[int, collections.deque[SendH2Data]]
+    stream_trailers: Dict[int, List[Tuple[bytes, bytes]]]
 
     def __init__(self, config: h2.config.H2Configuration):
         super().__init__(config)
         self.stream_buffers = collections.defaultdict(collections.deque)
+        self.stream_trailers = {}
 
     def send_data(
         self,
@@ -77,7 +93,16 @@ class BufferedH2Connection(h2.connection.H2Connection):
                 # We can't send right now, so we buffer.
                 self.stream_buffers[stream_id].append(SendH2Data(data, end_stream))
 
-    def end_stream(self, stream_id) -> None:
+    def send_trailers(self, stream_id: int, trailers: List[Tuple[bytes, bytes]]):
+        if self.stream_buffers.get(stream_id, None):
+            # Though trailers are not subject to flow control, we need to queue them and send strictly after data frames
+            self.stream_trailers[stream_id] = trailers
+        else:
+            self.send_headers(stream_id, trailers, end_stream=True)
+
+    def end_stream(self, stream_id: int) -> None:
+        if stream_id in self.stream_trailers:
+            return  # we already have trailers queued up that will end the stream.
         self.send_data(stream_id, b"", end_stream=True)
 
     def reset_stream(self, stream_id: int, error_code: int = 0) -> None:
@@ -147,6 +172,8 @@ class BufferedH2Connection(h2.connection.H2Connection):
             available_window -= len(chunk.data)
             if not self.stream_buffers[stream_id]:
                 del self.stream_buffers[stream_id]
+                if stream_id in self.stream_trailers:
+                    self.send_headers(stream_id, self.stream_trailers.pop(stream_id), end_stream=True)
             sent_any_data = True
 
         return sent_any_data
