@@ -4,11 +4,11 @@ from abc import ABCMeta
 from dataclasses import dataclass
 from typing import Optional
 
-from mitmproxy import connection, platform
-from mitmproxy.net import server_spec
+from mitmproxy import connection
 from mitmproxy.proxy import commands, events, layer
 from mitmproxy.proxy.commands import StartHook
-from mitmproxy.proxy.layers import tls
+from mitmproxy.proxy.layers import dns, tls
+from mitmproxy.proxy.mode_specs import ReverseMode
 from mitmproxy.proxy.utils import expect
 
 
@@ -20,12 +20,25 @@ class HttpProxy(layer.Layer):
         yield from child_layer.handle_event(event)
 
 
+class HttpUpstreamProxy(layer.Layer):
+    @expect(events.Start)
+    def _handle_event(self, event: events.Event) -> layer.CommandGenerator[None]:
+        child_layer = layer.NextLayer(self.context)
+        self._handle_event = child_layer.handle_event
+        yield from child_layer.handle_event(event)
+
+
 class DestinationKnown(layer.Layer, metaclass=ABCMeta):
     """Base layer for layers that gather connection destination info and then delegate."""
+
     child_layer: layer.Layer
 
     def finish_start(self) -> layer.CommandGenerator[Optional[str]]:
-        if self.context.options.connection_strategy == "eager" and self.context.server.address:
+        if (
+            self.context.options.connection_strategy == "eager"
+            and self.context.server.address
+            and self.context.server.transport_protocol == "tcp"
+        ):
             err = yield commands.OpenConnection(self.context.server)
             if err:
                 self._handle_event = self.done  # type: ignore
@@ -43,15 +56,20 @@ class DestinationKnown(layer.Layer, metaclass=ABCMeta):
 class ReverseProxy(DestinationKnown):
     @expect(events.Start)
     def _handle_event(self, event: events.Event) -> layer.CommandGenerator[None]:
-        spec = server_spec.parse_with_mode(self.context.options.mode)[1]
+        spec = self.context.client.proxy_mode
+        assert isinstance(spec, ReverseMode)
         self.context.server.address = spec.address
 
-        if spec.scheme not in ("http", "tcp"):
+        if spec.scheme == "https" or spec.scheme == "tls" or spec.scheme == "dtls":
             if not self.context.options.keep_host_header:
                 self.context.server.sni = spec.address[0]
             self.child_layer = tls.ServerTLSLayer(self.context)
-        else:
+        elif spec.scheme == "http" or spec.scheme == "tcp" or spec.scheme == "udp":
             self.child_layer = layer.NextLayer(self.context)
+        elif spec.scheme == "dns":
+            self.child_layer = dns.DNSLayer(self.context)
+        else:
+            raise AssertionError(self.context.client.transport_protocol)  # pragma: no cover
 
         err = yield from self.finish_start()
         if err:
@@ -61,15 +79,8 @@ class ReverseProxy(DestinationKnown):
 class TransparentProxy(DestinationKnown):
     @expect(events.Start)
     def _handle_event(self, event: events.Event) -> layer.CommandGenerator[None]:
-        assert platform.original_addr is not None
-        socket = yield commands.GetSocket(self.context.client)
-        try:
-            self.context.server.address = platform.original_addr(socket)
-        except Exception as e:
-            yield commands.Log(f"Transparent mode failure: {e!r}")
-
+        assert self.context.server.address
         self.child_layer = layer.NextLayer(self.context)
-
         err = yield from self.finish_start()
         if err:
             yield commands.CloseConnection(self.context.client)
@@ -105,6 +116,7 @@ class Socks5AuthHook(StartHook):
 
     This hook decides whether they are valid by setting `data.valid`.
     """
+
     data: Socks5AuthData
 
 
@@ -119,7 +131,8 @@ class Socks5Proxy(DestinationKnown):
         if reply_code is not None:
             yield commands.SendData(
                 self.context.client,
-                bytes([SOCKS5_VERSION, reply_code]) + b"\x00\x01\x00\x00\x00\x00\x00\x00"
+                bytes([SOCKS5_VERSION, reply_code])
+                + b"\x00\x01\x00\x00\x00\x00\x00\x00",
             )
         yield commands.CloseConnection(self.context.client)
         yield commands.Log(message)
@@ -134,7 +147,9 @@ class Socks5Proxy(DestinationKnown):
             yield from self.state()
         elif isinstance(event, events.ConnectionClosed):
             if self.buf:
-                yield commands.Log(f"Client closed connection before completing SOCKS5 handshake: {self.buf!r}")
+                yield commands.Log(
+                    f"Client closed connection before completing SOCKS5 handshake: {self.buf!r}"
+                )
             yield commands.CloseConnection(event.connection)
         else:
             raise AssertionError(f"Unknown event: {event}")
@@ -148,7 +163,9 @@ class Socks5Proxy(DestinationKnown):
                 guess = "Probably not a SOCKS request but a regular HTTP request. "
             else:
                 guess = ""
-            yield from self.socks_err(guess + "Invalid SOCKS version. Expected 0x05, got 0x%x" % self.buf[0])
+            yield from self.socks_err(
+                guess + "Invalid SOCKS version. Expected 0x05, got 0x%x" % self.buf[0]
+            )
             return
 
         n_methods = self.buf[1]
@@ -162,15 +179,19 @@ class Socks5Proxy(DestinationKnown):
             method = SOCKS5_METHOD_NO_AUTHENTICATION_REQUIRED
             self.state = self.state_connect
 
-        if method not in self.buf[2:2 + n_methods]:
-            method_str = "user/password" if method == SOCKS5_METHOD_USER_PASSWORD_AUTHENTICATION else "no"
+        if method not in self.buf[2 : 2 + n_methods]:
+            method_str = (
+                "user/password"
+                if method == SOCKS5_METHOD_USER_PASSWORD_AUTHENTICATION
+                else "no"
+            )
             yield from self.socks_err(
                 f"Client does not support SOCKS5 with {method_str} authentication.",
-                SOCKS5_METHOD_NO_ACCEPTABLE_METHODS
+                SOCKS5_METHOD_NO_ACCEPTABLE_METHODS,
             )
             return
         yield commands.SendData(self.context.client, bytes([SOCKS5_VERSION, method]))
-        self.buf = self.buf[2 + n_methods:]
+        self.buf = self.buf[2 + n_methods :]
         yield from self.state()
 
     state = state_greet
@@ -186,8 +207,10 @@ class Socks5Proxy(DestinationKnown):
         pass_len = self.buf[2 + user_len]
         if len(self.buf) < 3 + user_len + pass_len:
             return
-        user = self.buf[2:(2 + user_len)].decode("utf-8", "backslashreplace")
-        password = self.buf[(3 + user_len):(3 + user_len + pass_len)].decode("utf-8", "backslashreplace")
+        user = self.buf[2 : (2 + user_len)].decode("utf-8", "backslashreplace")
+        password = self.buf[(3 + user_len) : (3 + user_len + pass_len)].decode(
+            "utf-8", "backslashreplace"
+        )
 
         data = Socks5AuthData(self.context.client, user, password)
         yield Socks5AuthHook(data)
@@ -198,7 +221,7 @@ class Socks5Proxy(DestinationKnown):
             return
 
         yield commands.SendData(self.context.client, b"\x01\x00")
-        self.buf = self.buf[3 + user_len + pass_len:]
+        self.buf = self.buf[3 + user_len + pass_len :]
         self.state = self.state_connect
         yield from self.state()
 
@@ -208,7 +231,10 @@ class Socks5Proxy(DestinationKnown):
             return
 
         if self.buf[:3] != b"\x05\x01\x00":
-            yield from self.socks_err(f"Unsupported SOCKS5 request: {self.buf!r}", SOCKS5_REP_COMMAND_NOT_SUPPORTED)
+            yield from self.socks_err(
+                f"Unsupported SOCKS5 request: {self.buf!r}",
+                SOCKS5_REP_COMMAND_NOT_SUPPORTED,
+            )
             return
 
         # Determine message length
@@ -221,7 +247,9 @@ class Socks5Proxy(DestinationKnown):
         elif atyp == SOCKS5_ATYP_DOMAINNAME:
             message_len = 4 + 1 + self.buf[4] + 2
         else:
-            yield from self.socks_err(f"Unknown address type: {atyp}", SOCKS5_REP_ADDRESS_TYPE_NOT_SUPPORTED)
+            yield from self.socks_err(
+                f"Unknown address type: {atyp}", SOCKS5_REP_ADDRESS_TYPE_NOT_SUPPORTED
+            )
             return
 
         # Do we have enough bytes yet?
@@ -240,7 +268,7 @@ class Socks5Proxy(DestinationKnown):
             host_bytes = msg[5:-2]
             host = host_bytes.decode("ascii", "replace")
 
-        port, = struct.unpack("!H", msg[-2:])
+        (port,) = struct.unpack("!H", msg[-2:])
 
         # We now have all we need, let's get going.
         self.context.server.address = (host, port)
@@ -250,10 +278,16 @@ class Socks5Proxy(DestinationKnown):
         # but that's not a problem in practice...
         err = yield from self.finish_start()
         if err:
-            yield commands.SendData(self.context.client, b"\x05\x04\x00\x01\x00\x00\x00\x00\x00\x00")
+            yield commands.SendData(
+                self.context.client, b"\x05\x04\x00\x01\x00\x00\x00\x00\x00\x00"
+            )
             yield commands.CloseConnection(self.context.client)
         else:
-            yield commands.SendData(self.context.client, b"\x05\x00\x00\x01\x00\x00\x00\x00\x00\x00")
+            yield commands.SendData(
+                self.context.client, b"\x05\x00\x00\x01\x00\x00\x00\x00\x00\x00"
+            )
             if self.buf:
-                yield from self.child_layer.handle_event(events.DataReceived(self.context.client, self.buf))
+                yield from self.child_layer.handle_event(
+                    events.DataReceived(self.context.client, self.buf)
+                )
                 del self.buf
