@@ -1,8 +1,8 @@
-import typing
+import logging
+from collections.abc import Sequence
 
 from mitmproxy import exceptions
 from mitmproxy import flow
-from mitmproxy import ctx
 
 from mitmproxy.tools.console import overlay
 from mitmproxy.tools.console import signals
@@ -12,27 +12,25 @@ class CommandExecutor:
     def __init__(self, master):
         self.master = master
 
-    def __call__(self, cmd):
+    def __call__(self, cmd: str) -> None:
         if cmd.strip():
             try:
                 ret = self.master.commands.execute(cmd)
             except exceptions.CommandError as e:
-                ctx.log.error(str(e))
+                logging.error(str(e))
             else:
-                if ret:
-                    if type(ret) == typing.Sequence[flow.Flow]:
+                if ret is not None:
+                    if type(ret) == Sequence[flow.Flow]:
                         signals.status_message.send(
                             message="Command returned %s flows" % len(ret)
                         )
                     elif type(ret) == flow.Flow:
-                        signals.status_message.send(
-                            message="Command returned 1 flow"
-                        )
+                        signals.status_message.send(message="Command returned 1 flow")
                     else:
                         self.master.overlay(
                             overlay.DataViewerOverlay(
                                 self.master,
                                 ret,
                             ),
-                            valign="top"
+                            valign="top",
                         )
