@@ -1,8 +1,10 @@
 import enum
 import platform
-import typing
 import math
+from collections.abc import Iterable
 from functools import lru_cache
+from typing import Optional, Union
+
 from publicsuffix2 import get_sld, get_tld
 
 import urwid
@@ -12,14 +14,19 @@ from mitmproxy import flow
 from mitmproxy.http import HTTPFlow
 from mitmproxy.utils import human, emoji
 from mitmproxy.tcp import TCPFlow
+from mitmproxy.udp import UDPFlow
+from mitmproxy import dns
+from mitmproxy.dns import DNSFlow
 
 # Detect Windows Subsystem for Linux and Windows
-IS_WINDOWS = "Microsoft" in platform.platform() or "Windows" in platform.platform()
+IS_WINDOWS_OR_WSL = (
+    "Microsoft" in platform.platform() or "Windows" in platform.platform()
+)
 
 
 def is_keypress(k):
     """
-        Is this input event a keypress?
+    Is this input event a keypress?
     """
     if isinstance(k, str):
         return True
@@ -40,11 +47,11 @@ KEY_MAX = 30
 
 
 def format_keyvals(
-        entries: typing.Iterable[typing.Tuple[str, typing.Union[None, str, urwid.Widget]]],
-        key_format: str = "key",
-        value_format: str = "text",
-        indent: int = 0
-) -> typing.List[urwid.Columns]:
+    entries: Iterable[tuple[str, Union[None, str, urwid.Widget]]],
+    key_format: str = "key",
+    value_format: str = "text",
+    indent: int = 0,
+) -> list[urwid.Columns]:
     """
     Format a list of (key, value) tuples.
 
@@ -71,30 +78,18 @@ def format_keyvals(
             urwid.Columns(
                 [
                     ("fixed", indent, urwid.Text("")),
-                    (
-                        "fixed",
-                        max_key_len,
-                        urwid.Text([(key_format, k)])
-                    ),
-                    v
+                    ("fixed", max_key_len, urwid.Text([(key_format, k)])),
+                    v,
                 ],
-                dividechars=2
+                dividechars=2,
             )
         )
     return ret
 
 
-def fcol(s: str, attr: str) -> typing.Tuple[str, int, urwid.Text]:
+def fcol(s: str, attr: str) -> tuple[str, int, urwid.Text]:
     s = str(s)
-    return (
-        "fixed",
-        len(s),
-        urwid.Text(
-            [
-                (attr, s)
-            ]
-        )
-    )
+    return ("fixed", len(s), urwid.Text([(attr, s)]))
 
 
 if urwid.util.detected_encoding:
@@ -117,18 +112,20 @@ else:
     SYMBOL_TO_CLIENT = "<-"
 
 SCHEME_STYLES = {
-    'http': 'scheme_http',
-    'https': 'scheme_https',
-    'ws': 'scheme_ws',
-    'wss': 'scheme_wss',
-    'tcp': 'scheme_tcp',
+    "http": "scheme_http",
+    "https": "scheme_https",
+    "ws": "scheme_ws",
+    "wss": "scheme_wss",
+    "tcp": "scheme_tcp",
+    "udp": "scheme_udp",
+    "dns": "scheme_dns",
 }
 HTTP_REQUEST_METHOD_STYLES = {
-    'GET': 'method_get',
-    'POST': 'method_post',
-    'DELETE': 'method_delete',
-    'HEAD': 'method_head',
-    'PUT': 'method_put'
+    "GET": "method_get",
+    "POST": "method_post",
+    "DELETE": "method_delete",
+    "HEAD": "method_head",
+    "PUT": "method_put",
 }
 HTTP_RESPONSE_CODE_STYLE = {
     2: "code_200",
@@ -151,14 +148,14 @@ def fixlen(s: str, maxlen: int) -> str:
     if len(s) <= maxlen:
         return s.ljust(maxlen)
     else:
-        return s[0:maxlen - len(SYMBOL_ELLIPSIS)] + SYMBOL_ELLIPSIS
+        return s[0 : maxlen - len(SYMBOL_ELLIPSIS)] + SYMBOL_ELLIPSIS
 
 
 def fixlen_r(s: str, maxlen: int) -> str:
     if len(s) <= maxlen:
         return s.rjust(maxlen)
     else:
-        return SYMBOL_ELLIPSIS + s[len(s) - maxlen + len(SYMBOL_ELLIPSIS):]
+        return SYMBOL_ELLIPSIS + s[len(s) - maxlen + len(SYMBOL_ELLIPSIS) :]
 
 
 def render_marker(marker: str) -> str:
@@ -172,7 +169,7 @@ def render_marker(marker: str) -> str:
 
 
 class TruncatedText(urwid.Widget):
-    def __init__(self, text, attr, align='left'):
+    def __init__(self, text, attr, align="left"):
         self.text = text
         self.attr = attr
         self.align = align
@@ -187,11 +184,11 @@ class TruncatedText(urwid.Widget):
     def render(self, size, focus=False):
         text = self.text
         attr = self.attr
-        if self.align == 'right':
+        if self.align == "right":
             text = text[::-1]
             attr = attr[::-1]
 
-        text_len = len(text)  # TODO: unicode?
+        text_len = urwid.util.calc_width(text, 0, len(text))
         if size is not None and len(size) > 0:
             width = size[0]
         else:
@@ -200,26 +197,29 @@ class TruncatedText(urwid.Widget):
         if width >= text_len:
             remaining = width - text_len
             if remaining > 0:
-                c_text = text + ' ' * remaining
-                c_attr = attr + [('text', remaining)]
+                c_text = text + " " * remaining
+                c_attr = attr + [("text", remaining)]
             else:
                 c_text = text
                 c_attr = attr
         else:
-            visible_len = width - len(SYMBOL_ELLIPSIS)
-            visible_text = text[0:visible_len]
+            trim = urwid.util.calc_trim_text(text, 0, width - 1, 0, width - 1)
+            visible_text = text[0 : trim[1]]
+            if trim[3] == 1:
+                visible_text += " "
             c_text = visible_text + SYMBOL_ELLIPSIS
-            c_attr = (urwid.util.rle_subseg(attr, 0, len(visible_text.encode())) +
-                      [('focus', len(SYMBOL_ELLIPSIS.encode()))])
+            c_attr = urwid.util.rle_subseg(attr, 0, len(visible_text.encode())) + [
+                ("focus", len(SYMBOL_ELLIPSIS.encode()))
+            ]
 
-        if self.align == 'right':
+        if self.align == "right":
             c_text = c_text[::-1]
             c_attr = c_attr[::-1]
 
         return urwid.TextCanvas([c_text.encode()], [c_attr], maxcol=width)
 
 
-def truncated_plain(text, attr, align='left'):
+def truncated_plain(text, attr, align="left"):
     return TruncatedText(text, [(attr, len(text.encode()))], align)
 
 
@@ -254,106 +254,111 @@ def colorize_host(host):
     for letter in reversed(range(len(host))):
         character = host[letter]
         if tld_size > 0:
-            style = 'url_domain'
+            style = "url_domain"
             tld_size -= 1
         elif tld_size == 0:
-            style = 'text'
+            style = "text"
             tld_size -= 1
         elif sld_size > 0:
             sld_size -= 1
-            style = 'url_extension'
+            style = "url_extension"
         else:
-            style = 'text'
+            style = "text"
         rle_append_beginning_modify(attr, (style, len(character.encode())))
     return attr
 
 
 def colorize_req(s):
-    path = s.split('?', 2)[0]
+    path = s.split("?", 2)[0]
     i_query = len(path)
-    i_last_slash = path.rfind('/')
-    i_ext = path[i_last_slash + 1:].rfind('.')
+    i_last_slash = path.rfind("/")
+    i_ext = path[i_last_slash + 1 :].rfind(".")
     i_ext = i_last_slash + i_ext if i_ext >= 0 else len(s)
     in_val = False
     attr = []
     for i in range(len(s)):
         c = s[i]
-        if ((i < i_query and c == '/') or
-                (i < i_query and i > i_last_slash and c == '.') or
-                (i == i_query)):
-            a = 'url_punctuation'
+        if (
+            (i < i_query and c == "/")
+            or (i < i_query and i > i_last_slash and c == ".")
+            or (i == i_query)
+        ):
+            a = "url_punctuation"
         elif i > i_query:
             if in_val:
-                if c == '&':
+                if c == "&":
                     in_val = False
-                    a = 'url_punctuation'
+                    a = "url_punctuation"
                 else:
-                    a = 'url_query_value'
+                    a = "url_query_value"
             else:
-                if c == '=':
+                if c == "=":
                     in_val = True
-                    a = 'url_punctuation'
+                    a = "url_punctuation"
                 else:
-                    a = 'url_query_key'
+                    a = "url_query_key"
         elif i > i_ext:
-            a = 'url_extension'
+            a = "url_extension"
         elif i > i_last_slash:
-            a = 'url_filename'
+            a = "url_filename"
         else:
-            a = 'text'
+            a = "text"
         urwid.util.rle_append_modify(attr, (a, len(c.encode())))
     return attr
 
 
 def colorize_url(url):
-    parts = url.split('/', 3)
-    if len(parts) < 4 or len(parts[1]) > 0 or parts[0][-1:] != ':':
-        return [('error', len(url))]  # bad URL
-    return [
-               (SCHEME_STYLES.get(parts[0], "scheme_other"), len(parts[0]) - 1),
-               ('url_punctuation', 3),  # ://
-           ] + colorize_host(parts[2]) + colorize_req('/' + parts[3])
+    parts = url.split("/", 3)
+    if len(parts) < 4 or len(parts[1]) > 0 or parts[0][-1:] != ":":
+        return [("error", len(url))]  # bad URL
+    return (
+        [
+            (SCHEME_STYLES.get(parts[0], "scheme_other"), len(parts[0]) - 1),
+            ("url_punctuation", 3),  # ://
+        ]
+        + colorize_host(parts[2])
+        + colorize_req("/" + parts[3])
+    )
 
 
-def format_http_content_type(content_type: str) -> typing.Tuple[str, str]:
+def format_http_content_type(content_type: str) -> tuple[str, str]:
     content_type = content_type.split(";")[0]
-    if content_type.endswith('/javascript'):
-        style = 'content_script'
-    elif content_type.startswith('text/'):
-        style = 'content_text'
-    elif (content_type.startswith('image/') or
-          content_type.startswith('video/') or
-          content_type.startswith('font/') or
-          "/x-font-" in content_type):
-        style = 'content_media'
-    elif content_type.endswith('/json') or content_type.endswith('/xml'):
-        style = 'content_data'
-    elif content_type.startswith('application/'):
-        style = 'content_raw'
+    if content_type.endswith("/javascript"):
+        style = "content_script"
+    elif content_type.startswith("text/"):
+        style = "content_text"
+    elif (
+        content_type.startswith("image/")
+        or content_type.startswith("video/")
+        or content_type.startswith("font/")
+        or "/x-font-" in content_type
+    ):
+        style = "content_media"
+    elif content_type.endswith("/json") or content_type.endswith("/xml"):
+        style = "content_data"
+    elif content_type.startswith("application/"):
+        style = "content_raw"
     else:
-        style = 'content_other'
+        style = "content_other"
     return content_type, style
 
 
-def format_duration(duration: float) -> typing.Tuple[str, str]:
+def format_duration(duration: float) -> tuple[str, str]:
     pretty_duration = human.pretty_duration(duration)
-    style = 'gradient_%02d' % int(99 - 100 * min(math.log2(1 + 1000 * duration) / 12, 0.99))
+    style = "gradient_%02d" % int(
+        99 - 100 * min(math.log2(1 + 1000 * duration) / 12, 0.99)
+    )
     return pretty_duration, style
 
 
-def format_size(num_bytes: int) -> typing.Tuple[str, str]:
+def format_size(num_bytes: int) -> tuple[str, str]:
     pretty_size = human.pretty_size(num_bytes)
-    style = 'gradient_%02d' % int(99 - 100 * min(math.log2(1 + num_bytes) / 20, 0.99))
+    style = "gradient_%02d" % int(99 - 100 * min(math.log2(1 + num_bytes) / 20, 0.99))
     return pretty_size, style
 
 
-def format_left_indicators(
-        *,
-        focused: bool,
-        intercepted: bool,
-        timestamp: float
-):
-    indicators: typing.List[typing.Union[str, typing.Tuple[str, str]]] = []
+def format_left_indicators(*, focused: bool, intercepted: bool, timestamp: float):
+    indicators: list[Union[str, tuple[str, str]]] = []
     if focused:
         indicators.append(("focus", ">>"))
     else:
@@ -367,11 +372,11 @@ def format_left_indicators(
 
 
 def format_right_indicators(
-        *,
-        replay: bool,
-        marked: str,
+    *,
+    replay: bool,
+    marked: str,
 ):
-    indicators: typing.List[typing.Union[str, typing.Tuple[str, str]]] = []
+    indicators: list[Union[str, tuple[str, str]]] = []
     if replay:
         indicators.append(("replay", SYMBOL_REPLAY))
     else:
@@ -385,26 +390,26 @@ def format_right_indicators(
 
 @lru_cache(maxsize=800)
 def format_http_flow_list(
-        *,
-        render_mode: RenderMode,
-        focused: bool,
-        marked: str,
-        is_replay: bool,
-        request_method: str,
-        request_scheme: str,
-        request_host: str,
-        request_path: str,
-        request_url: str,
-        request_http_version: str,
-        request_timestamp: float,
-        request_is_push_promise: bool,
-        intercepted: bool,
-        response_code: typing.Optional[int],
-        response_reason: typing.Optional[str],
-        response_content_length: typing.Optional[int],
-        response_content_type: typing.Optional[str],
-        duration: typing.Optional[float],
-        error_message: typing.Optional[str],
+    *,
+    render_mode: RenderMode,
+    focused: bool,
+    marked: str,
+    is_replay: bool,
+    request_method: str,
+    request_scheme: str,
+    request_host: str,
+    request_path: str,
+    request_url: str,
+    request_http_version: str,
+    request_timestamp: float,
+    request_is_push_promise: bool,
+    intercepted: bool,
+    response_code: Optional[int],
+    response_reason: Optional[str],
+    response_content_length: Optional[int],
+    response_content_type: Optional[str],
+    duration: Optional[float],
+    error_message: Optional[str],
 ) -> urwid.Widget:
     req = []
 
@@ -420,7 +425,7 @@ def format_http_flow_list(
     req.append(fcol(request_method, method_style))
 
     if request_is_push_promise:
-        req.append(fcol('PUSH_PROMISE', 'method_http2_push'))
+        req.append(fcol("PUSH_PROMISE", "method_http2_push"))
 
     preamble_len = sum(x[1] for x in req) + len(req) - 1
 
@@ -434,24 +439,22 @@ def format_http_flow_list(
         url_style = "title"
 
     if render_mode is RenderMode.DETAILVIEW:
-        req.append(
-            urwid.Text([(url_style, request_url)])
-        )
+        req.append(urwid.Text([(url_style, request_url)]))
     else:
         req.append(truncated_plain(request_url, url_style))
 
     req.append(format_right_indicators(replay=is_replay, marked=marked))
 
-    resp = [
-        ("fixed", preamble_len, urwid.Text(""))
-    ]
+    resp = [("fixed", preamble_len, urwid.Text(""))]
     if response_code:
         if intercepted:
             style = "intercept"
         else:
             style = ""
 
-        status_style = style or HTTP_RESPONSE_CODE_STYLE.get(response_code // 100, "code_other")
+        status_style = style or HTTP_RESPONSE_CODE_STYLE.get(
+            response_code // 100, "code_other"
+        )
         resp.append(fcol(SYMBOL_RETURN, status_style))
         resp.append(fcol(str(response_code), status_style))
         if response_reason and render_mode is RenderMode.DETAILVIEW:
@@ -478,40 +481,37 @@ def format_http_flow_list(
         resp.append(fcol(SYMBOL_RETURN, "error"))
         resp.append(urwid.Text([("error", error_message)]))
 
-    return urwid.Pile([
-        urwid.Columns(req, dividechars=1),
-        urwid.Columns(resp, dividechars=1)
-    ])
+    return urwid.Pile(
+        [urwid.Columns(req, dividechars=1), urwid.Columns(resp, dividechars=1)]
+    )
 
 
 @lru_cache(maxsize=800)
 def format_http_flow_table(
-        *,
-        render_mode: RenderMode,
-        focused: bool,
-        marked: str,
-        is_replay: typing.Optional[str],
-        request_method: str,
-        request_scheme: str,
-        request_host: str,
-        request_path: str,
-        request_url: str,
-        request_http_version: str,
-        request_timestamp: float,
-        request_is_push_promise: bool,
-        intercepted: bool,
-        response_code: typing.Optional[int],
-        response_reason: typing.Optional[str],
-        response_content_length: typing.Optional[int],
-        response_content_type: typing.Optional[str],
-        duration: typing.Optional[float],
-        error_message: typing.Optional[str],
+    *,
+    render_mode: RenderMode,
+    focused: bool,
+    marked: str,
+    is_replay: Optional[str],
+    request_method: str,
+    request_scheme: str,
+    request_host: str,
+    request_path: str,
+    request_url: str,
+    request_http_version: str,
+    request_timestamp: float,
+    request_is_push_promise: bool,
+    intercepted: bool,
+    response_code: Optional[int],
+    response_reason: Optional[str],
+    response_content_length: Optional[int],
+    response_content_type: Optional[str],
+    duration: Optional[float],
+    error_message: Optional[str],
 ) -> urwid.Widget:
     items = [
         format_left_indicators(
-            focused=focused,
-            intercepted=intercepted,
-            timestamp=request_timestamp
+            focused=focused, intercepted=intercepted, timestamp=request_timestamp
         )
     ]
 
@@ -524,13 +524,23 @@ def format_http_flow_table(
     items.append(fcol(fixlen(request_scheme.upper(), 5), scheme_style))
 
     if request_is_push_promise:
-        method_style = 'method_http2_push'
+        method_style = "method_http2_push"
     else:
-        method_style = request_style or HTTP_REQUEST_METHOD_STYLES.get(request_method, "method_other")
+        method_style = request_style or HTTP_REQUEST_METHOD_STYLES.get(
+            request_method, "method_other"
+        )
     items.append(fcol(fixlen(request_method, 4), method_style))
 
-    items.append(('weight', 0.25, TruncatedText(request_host, colorize_host(request_host), 'right')))
-    items.append(('weight', 1.0, TruncatedText(request_path, colorize_req(request_path), 'left')))
+    items.append(
+        (
+            "weight",
+            0.25,
+            TruncatedText(request_host, colorize_host(request_host), "right"),
+        )
+    )
+    items.append(
+        ("weight", 1.0, TruncatedText(request_path, colorize_req(request_path), "left"))
+    )
 
     if intercepted and response_code:
         response_style = "intercept"
@@ -540,35 +550,37 @@ def format_http_flow_table(
     if response_code:
 
         status = str(response_code)
-        status_style = response_style or HTTP_RESPONSE_CODE_STYLE.get(response_code // 100, "code_other")
+        status_style = response_style or HTTP_RESPONSE_CODE_STYLE.get(
+            response_code // 100, "code_other"
+        )
 
         if response_content_length and response_content_type:
             content, content_style = format_http_content_type(response_content_type)
             content_style = response_style or content_style
         elif response_content_length:
-            content = ''
-            content_style = 'content_none'
+            content = ""
+            content_style = "content_none"
         elif response_content_length == 0:
             content = "[no content]"
-            content_style = 'content_none'
+            content_style = "content_none"
         else:
             content = "[content missing]"
-            content_style = 'content_none'
+            content_style = "content_none"
 
     elif error_message:
-        status = 'err'
-        status_style = 'error'
+        status = "err"
+        status_style = "error"
         content = error_message
-        content_style = 'error'
+        content_style = "error"
 
     else:
-        status = ''
-        status_style = 'text'
-        content = ''
-        content_style = ''
+        status = ""
+        status_style = "text"
+        content = ""
+        content_style = ""
 
     items.append(fcol(fixlen(status, 3), status_style))
-    items.append(('weight', 0.15, truncated_plain(content, content_style, 'right')))
+    items.append(("weight", 0.15, truncated_plain(content, content_style, "right")))
 
     if response_content_length:
         size, size_style = format_size(response_content_length)
@@ -578,29 +590,34 @@ def format_http_flow_table(
 
     if duration:
         duration_pretty, duration_style = format_duration(duration)
-        items.append(fcol(fixlen_r(duration_pretty, 5), response_style or duration_style))
+        items.append(
+            fcol(fixlen_r(duration_pretty, 5), response_style or duration_style)
+        )
     else:
         items.append(("fixed", 5, urwid.Text("")))
 
-    items.append(format_right_indicators(
-        replay=bool(is_replay),
-        marked=marked,
-    ))
+    items.append(
+        format_right_indicators(
+            replay=bool(is_replay),
+            marked=marked,
+        )
+    )
     return urwid.Columns(items, dividechars=1, min_width=15)
 
 
 @lru_cache(maxsize=800)
-def format_tcp_flow(
-        *,
-        render_mode: RenderMode,
-        focused: bool,
-        timestamp_start: float,
-        marked: str,
-        client_address,
-        server_address,
-        total_size: int,
-        duration: typing.Optional[float],
-        error_message: typing.Optional[str],
+def format_message_flow(
+    *,
+    render_mode: RenderMode,
+    focused: bool,
+    timestamp_start: float,
+    marked: str,
+    protocol: str,
+    client_address,
+    server_address,
+    total_size: int,
+    duration: Optional[float],
+    error_message: Optional[str],
 ):
     conn = f"{human.format_address(client_address)} <-> {human.format_address(server_address)}"
 
@@ -608,7 +625,9 @@ def format_tcp_flow(
 
     if render_mode in (RenderMode.TABLE, RenderMode.DETAILVIEW):
         items.append(
-            format_left_indicators(focused=focused, intercepted=False, timestamp=timestamp_start)
+            format_left_indicators(
+                focused=focused, intercepted=False, timestamp=timestamp_start
+            )
         )
     else:
         if focused:
@@ -617,13 +636,13 @@ def format_tcp_flow(
             items.append(fcol("  ", "focus"))
 
     if render_mode is RenderMode.TABLE:
-        items.append(fcol("TCP  ", SCHEME_STYLES["tcp"]))
+        items.append(fcol(fixlen(protocol.upper(), 5), SCHEME_STYLES[protocol]))
     else:
-        items.append(fcol("TCP", SCHEME_STYLES["tcp"]))
+        items.append(fcol(protocol.upper(), SCHEME_STYLES[protocol]))
 
-    items.append(('weight', 1.0, truncated_plain(conn, "text", 'left')))
+    items.append(("weight", 1.0, truncated_plain(conn, "text", "left")))
     if error_message:
-        items.append(('weight', 1.0, truncated_plain(error_message, "error", 'left')))
+        items.append(("weight", 1.0, truncated_plain(error_message, "error", "left")))
 
     if total_size:
         size, size_style = format_size(total_size)
@@ -639,17 +658,89 @@ def format_tcp_flow(
 
     items.append(format_right_indicators(replay=False, marked=marked))
 
-    return urwid.Pile([
-        urwid.Columns(items, dividechars=1, min_width=15)
-    ])
+    return urwid.Pile([urwid.Columns(items, dividechars=1, min_width=15)])
+
+
+@lru_cache(maxsize=800)
+def format_dns_flow(
+    *,
+    render_mode: RenderMode,
+    focused: bool,
+    intercepted: bool,
+    marked: str,
+    is_replay: Optional[str],
+    op_code: str,
+    request_timestamp: float,
+    domain: str,
+    type: str,
+    response_code: Optional[str],
+    response_code_http_equiv: int,
+    answer: Optional[str],
+    error_message: str,
+    duration: Optional[float],
+):
+    items = []
+
+    if render_mode in (RenderMode.TABLE, RenderMode.DETAILVIEW):
+        items.append(
+            format_left_indicators(
+                focused=focused, intercepted=intercepted, timestamp=request_timestamp
+            )
+        )
+    else:
+        items.append(fcol(">>" if focused else "  ", "focus"))
+
+    scheme_style = "intercepted" if intercepted else SCHEME_STYLES["dns"]
+    t = f"DNS {op_code}"
+    if render_mode is RenderMode.TABLE:
+        t = fixlen(t, 10)
+    items.append(fcol(t, scheme_style))
+    items.append(("weight", 0.5, TruncatedText(domain, colorize_host(domain), "right")))
+    items.append(fcol("(" + fixlen(type, 5)[: len(type)] + ") =", "text"))
+
+    items.append(
+        (
+            "weight",
+            1,
+            (
+                truncated_plain(
+                    "..." if answer is None else "?" if not answer else answer, "text"
+                )
+                if error_message is None
+                else truncated_plain(error_message, "error")
+            ),
+        )
+    )
+    status_style = (
+        "intercepted"
+        if intercepted
+        else HTTP_RESPONSE_CODE_STYLE.get(response_code_http_equiv // 100, "code_other")
+    )
+    items.append(
+        fcol(fixlen("" if response_code is None else response_code, 9), status_style)
+    )
+
+    if duration:
+        duration_pretty, duration_style = format_duration(duration)
+        items.append(fcol(fixlen_r(duration_pretty, 5), duration_style))
+    else:
+        items.append(("fixed", 5, urwid.Text("")))
+
+    items.append(
+        format_right_indicators(
+            replay=bool(is_replay),
+            marked=marked,
+        )
+    )
+    return urwid.Pile([urwid.Columns(items, dividechars=1, min_width=15)])
 
 
 def format_flow(
-        f: flow.Flow,
-        *,
-        render_mode: RenderMode,
-        hostheader: bool = False,  # pass options directly if we need more stuff from them
-        focused: bool = True,
+    f: flow.Flow,
+    *,
+    render_mode: RenderMode,
+    hostheader: bool = False,  # pass options directly if we need more stuff from them
+    focused: bool = True,
 ) -> urwid.Widget:
     """
     This functions calls the proper renderer depending on the flow type.
@@ -657,47 +748,81 @@ def format_flow(
     relevant for display and call the render with only that. This assures that rows
     are updated if the flow is changed.
     """
-    duration: typing.Optional[float]
-    error_message: typing.Optional[str]
+    duration: Optional[float]
+    error_message: Optional[str]
     if f.error:
         error_message = f.error.msg
     else:
         error_message = None
 
-    if isinstance(f, TCPFlow):
+    if isinstance(f, (TCPFlow, UDPFlow)):
         total_size = 0
         for message in f.messages:
             total_size += len(message.content)
         if f.messages:
-            duration = f.messages[-1].timestamp - f.timestamp_start
+            duration = f.messages[-1].timestamp - f.client_conn.timestamp_start
         else:
             duration = None
-        return format_tcp_flow(
+        return format_message_flow(
             render_mode=render_mode,
             focused=focused,
-            timestamp_start=f.timestamp_start,
+            timestamp_start=f.client_conn.timestamp_start,
             marked=f.marked,
+            protocol=f.type,
             client_address=f.client_conn.peername,
             server_address=f.server_conn.address,
             total_size=total_size,
             duration=duration,
             error_message=error_message,
         )
-    elif isinstance(f, HTTPFlow):
-        intercepted = (
-                f.intercepted and not (f.reply and f.reply.state == "committed")
+    elif isinstance(f, DNSFlow):
+        if f.response:
+            duration = f.response.timestamp - f.request.timestamp
+            response_code_str: Optional[str] = dns.response_codes.to_str(
+                f.response.response_code
+            )
+            response_code_http_equiv = dns.response_codes.http_equiv_status_code(
+                f.response.response_code
+            )
+            answer = ", ".join(str(x) for x in f.response.answers)
+        else:
+            duration = None
+            response_code_str = None
+            response_code_http_equiv = 0
+            answer = None
+        return format_dns_flow(
+            render_mode=render_mode,
+            focused=focused,
+            intercepted=f.intercepted,
+            marked=f.marked,
+            is_replay=f.is_replay,
+            op_code=dns.op_codes.to_str(f.request.op_code),
+            request_timestamp=f.request.timestamp,
+            domain=f.request.questions[0].name if f.request.questions else "",
+            type=dns.types.to_str(f.request.questions[0].type)
+            if f.request.questions
+            else "",
+            response_code=response_code_str,
+            response_code_http_equiv=response_code_http_equiv,
+            answer=answer,
+            error_message=error_message,
+            duration=duration,
         )
-        response_content_length: typing.Optional[int]
+    elif isinstance(f, HTTPFlow):
+        intercepted = f.intercepted
+        response_content_length: Optional[int]
         if f.response:
             if f.response.raw_content is not None:
                 response_content_length = len(f.response.raw_content)
             else:
                 response_content_length = None
-            response_code: typing.Optional[int] = f.response.status_code
-            response_reason: typing.Optional[str] = f.response.reason
+            response_code: Optional[int] = f.response.status_code
+            response_reason: Optional[str] = f.response.reason
             response_content_type = f.response.headers.get("content-type")
             if f.response.timestamp_end:
-                duration = max([f.response.timestamp_end - f.request.timestamp_start, 0])
+                duration = max(
+                    [f.response.timestamp_end - f.request.timestamp_start, 0]
+                )
             else:
                 duration = None
         else:
@@ -730,7 +855,7 @@ def format_flow(
             request_url=f.request.pretty_url if hostheader else f.request.url,
             request_http_version=f.request.http_version,
             request_timestamp=f.request.timestamp_start,
-            request_is_push_promise='h2-pushed-stream' in f.metadata,
+            request_is_push_promise="h2-pushed-stream" in f.metadata,
             intercepted=intercepted,
             response_code=response_code,
             response_reason=response_reason,
