@@ -1,8 +1,10 @@
+from __future__ import annotations
+from collections.abc import Sequence
+
 import urwid
-import blinker
 import textwrap
 import pprint
-from typing import Optional, Sequence
+from typing import Optional
 
 from mitmproxy import exceptions
 from mitmproxy import optmanager
@@ -22,14 +24,7 @@ def can_edit_inplace(opt):
 
 def fcol(s, width, attr):
     s = str(s)
-    return (
-        "fixed",
-        width,
-        urwid.Text((attr, s))
-    )
-
-
-option_focus_change = blinker.Signal()
+    return ("fixed", width, urwid.Text((attr, s)))
 
 
 class OptionItem(urwid.WidgetWrap):
@@ -61,22 +56,19 @@ class OptionItem(urwid.WidgetWrap):
             valw = urwid.Edit(edit_text=displayval)
         else:
             valw = urwid.AttrMap(
-                urwid.Padding(
-                    urwid.Text([(valstyle, displayval)])
-                ),
-                valstyle
+                urwid.Padding(urwid.Text([(valstyle, displayval)])), valstyle
             )
 
         return urwid.Columns(
             [
                 (
                     self.namewidth,
-                    urwid.Text([("title", self.opt.name.ljust(self.namewidth))])
+                    urwid.Text([("title", self.opt.name.ljust(self.namewidth))]),
                 ),
-                valw
+                valw,
             ],
             dividechars=2,
-            focus_column=1
+            focus_column=1,
         )
 
     def get_edit_text(self):
@@ -93,8 +85,9 @@ class OptionItem(urwid.WidgetWrap):
 
 
 class OptionListWalker(urwid.ListWalker):
-    def __init__(self, master):
+    def __init__(self, master, help_widget: OptionHelp):
         self.master = master
+        self.help_widget = help_widget
 
         self.index = 0
         self.focusobj = None
@@ -128,9 +121,7 @@ class OptionListWalker(urwid.ListWalker):
     def _get(self, pos, editing):
         name = self.opts[pos]
         opt = self.master.options._options[name]
-        return OptionItem(
-            self, opt, pos == self.index, self.maxlen, editing
-        )
+        return OptionItem(self, opt, pos == self.index, self.maxlen, editing)
 
     def get_focus(self):
         return self.focus_obj, self.index
@@ -141,7 +132,7 @@ class OptionListWalker(urwid.ListWalker):
         opt = self.master.options._options[name]
         self.index = index
         self.focus_obj = self._get(self.index, self.editing)
-        option_focus_change.send(opt.help)
+        self.help_widget.update_help_text(opt.help)
         self._modified()
 
     def get_next(self, pos):
@@ -164,9 +155,9 @@ class OptionListWalker(urwid.ListWalker):
 
 
 class OptionsList(urwid.ListBox):
-    def __init__(self, master):
+    def __init__(self, master, help_widget: OptionHelp):
         self.master = master
-        self.walker = OptionListWalker(master)
+        self.walker = OptionListWalker(master, help_widget)
         super().__init__(self.walker)
 
     def save_config(self, path):
@@ -181,8 +172,7 @@ class OptionsList(urwid.ListBox):
                 foc, idx = self.get_focus()
                 v = self.walker.get_edit_text()
                 try:
-                    d = self.master.options.parse_setval(foc.opt, v)
-                    self.master.options.update(**{foc.opt.name: d})
+                    self.master.options.set(f"{foc.opt.name}={v}")
                 except exceptions.OptionsError as v:
                     signals.status_message.send(message=str(v))
                 self.walker.stop_editing()
@@ -213,7 +203,7 @@ class OptionsList(urwid.ListBox):
                             foc.opt.name,
                             foc.opt.choices,
                             foc.opt.current(),
-                            self.master.options.setter(foc.opt.name)
+                            self.master.options.setter(foc.opt.name),
                         )
                     )
                 elif foc.opt.typespec == Sequence[str]:
@@ -222,9 +212,9 @@ class OptionsList(urwid.ListBox):
                             self.master,
                             foc.opt.name,
                             foc.opt.current(),
-                            HELP_HEIGHT + 5
+                            HELP_HEIGHT + 5,
                         ),
-                        valign="top"
+                        valign="top",
                     )
                 else:
                     raise NotImplementedError()
@@ -236,7 +226,6 @@ class OptionHelp(urwid.Frame):
         self.master = master
         super().__init__(self.widget(""))
         self.set_active(False)
-        option_focus_change.connect(self.sig_mod)
 
     def set_active(self, val):
         h = urwid.Text("Option Help")
@@ -245,11 +234,9 @@ class OptionHelp(urwid.Frame):
 
     def widget(self, txt):
         cols, _ = self.master.ui.get_cols_rows()
-        return urwid.ListBox(
-            [urwid.Text(i) for i in textwrap.wrap(txt, cols)]
-        )
+        return urwid.ListBox([urwid.Text(i) for i in textwrap.wrap(txt, cols)])
 
-    def sig_mod(self, txt):
+    def update_help_text(self, txt: str) -> None:
         self.set_body(self.widget(txt))
 
 
@@ -259,7 +246,7 @@ class Options(urwid.Pile, layoutwidget.LayoutWidget):
 
     def __init__(self, master):
         oh = OptionHelp(master)
-        self.optionslist = OptionsList(master)
+        self.optionslist = OptionsList(master, oh)
         super().__init__(
             [
                 self.optionslist,
@@ -274,9 +261,7 @@ class Options(urwid.Pile, layoutwidget.LayoutWidget):
 
     def keypress(self, size, key):
         if key == "m_next":
-            self.focus_position = (
-                self.focus_position + 1
-            ) % len(self.widget_list)
+            self.focus_position = (self.focus_position + 1) % len(self.widget_list)
             self.widget_list[1].set_active(self.focus_position == 1)
             key = None
 
@@ -284,7 +269,7 @@ class Options(urwid.Pile, layoutwidget.LayoutWidget):
         # So much for "closed for modification, but open for extension".
         item_rows = None
         if len(size) == 2:
-            item_rows = self.get_item_rows(size, focus = True)
+            item_rows = self.get_item_rows(size, focus=True)
         i = self.widget_list.index(self.focus_item)
         tsize = self.get_item_size(size, i, True, item_rows)
         return self.focus_item.keypress(tsize, key)
