@@ -7,9 +7,11 @@ import signal
 import sys
 import threading
 import traceback
+from collections import Counter
 from contextlib import redirect_stdout
 
 from OpenSSL import SSL
+
 from mitmproxy import version
 from mitmproxy.utils import asyncio_utils
 
@@ -20,13 +22,13 @@ def dump_system_info():
     data = [
         f"Mitmproxy: {mitmproxy_version}",
         f"Python:    {platform.python_version()}",
-        "OpenSSL:   {}".format(SSL.SSLeay_version(SSL.SSLEAY_VERSION).decode()),
+        f"OpenSSL:   {SSL.SSLeay_version(SSL.SSLEAY_VERSION).decode()}",
         f"Platform:  {platform.platform()}",
     ]
     return "\n".join(data)
 
 
-def dump_info(signal=None, frame=None, file=sys.stdout, testing=False):  # pragma: no cover
+def dump_info(signal=None, frame=None, file=sys.stdout):  # pragma: no cover
     with redirect_stdout(file):
         print("****************************************************")
         print("Summary")
@@ -70,17 +72,19 @@ def dump_info(signal=None, frame=None, file=sys.stdout, testing=False):  # pragm
 
         print()
         print("Memory")
-        print("=======")
+        print("======")
         gc.collect()
-        d = {}
-        for i in gc.get_objects():
-            t = str(type(i))
-            if "mitmproxy" in t:
-                d[t] = d.setdefault(t, 0) + 1
-        itms = list(d.items())
-        itms.sort(key=lambda x: x[1])
-        for i in itms[-20:]:
-            print(i[1], i[0])
+        objs = Counter(str(type(i)) for i in gc.get_objects())
+
+        for cls, count in objs.most_common(20):
+            print(f"{count} {cls}")
+
+        print()
+        print("Memory (mitmproxy only)")
+        print("=======================")
+        mitm_objs = Counter({k: v for k, v in objs.items() if "mitmproxy" in k})
+        for cls, count in mitm_objs.most_common(20):
+            print(f"{count} {cls}")
 
         try:
             asyncio.get_running_loop()
@@ -92,32 +96,29 @@ def dump_info(signal=None, frame=None, file=sys.stdout, testing=False):  # pragm
             print("=======")
             for task in asyncio.all_tasks():
                 f = task.get_stack(limit=1)[0]
-                line = linecache.getline(f.f_code.co_filename, f.f_lineno, f.f_globals).strip()
+                line = linecache.getline(
+                    f.f_code.co_filename, f.f_lineno, f.f_globals
+                ).strip()
                 line = f"{line}  # at {os.path.basename(f.f_code.co_filename)}:{f.f_lineno}"
-                print(f"{asyncio_utils.task_repr(task)}\n"
-                      f"    {line}")
+                print(f"{asyncio_utils.task_repr(task)}\n" f"    {line}")
 
         print("****************************************************")
 
-    if not testing:
+    if os.getenv("MITMPROXY_DEBUG_EXIT"):  # pragma: no cover
         sys.exit(1)
 
 
-def dump_stacks(signal=None, frame=None, file=sys.stdout, testing=False):
+def dump_stacks(signal=None, frame=None, file=sys.stdout):
     id2name = {th.ident: th.name for th in threading.enumerate()}
     code = []
     for threadId, stack in sys._current_frames().items():
-        code.append(
-            "\n# Thread: %s(%d)" % (
-                id2name.get(threadId, ""), threadId
-            )
-        )
+        code.append("\n# Thread: %s(%d)" % (id2name.get(threadId, ""), threadId))
         for filename, lineno, name, line in traceback.extract_stack(stack):
             code.append('File: "%s", line %d, in %s' % (filename, lineno, name))
             if line:
                 code.append("  %s" % (line.strip()))
     print("\n".join(code), file=file)
-    if not testing:  # pragma: no cover
+    if os.getenv("MITMPROXY_DEBUG_EXIT"):  # pragma: no cover
         sys.exit(1)
 
 
