@@ -1,46 +1,28 @@
 import typing
+from collections import abc
+
+try:
+    from types import UnionType
+except ImportError:  # pragma: no cover
+    UnionType = object()  # type: ignore
 
 Type = typing.Union[
     typing.Any  # anything more elaborate really fails with mypy at the moment.
 ]
 
 
-def sequence_type(typeinfo: typing.Type[typing.List]) -> Type:
-    """Return the type of a sequence, e.g. typing.List"""
-    return typeinfo.__args__[0]  # type: ignore
-
-
-def tuple_types(typeinfo: typing.Type[typing.Tuple]) -> typing.Sequence[Type]:
-    """Return the types of a typing.Tuple"""
-    return typeinfo.__args__  # type: ignore
-
-
-def union_types(typeinfo: typing.Type[typing.Tuple]) -> typing.Sequence[Type]:
-    """return the types of a typing.Union"""
-    return typeinfo.__args__  # type: ignore
-
-
-def mapping_types(typeinfo: typing.Type[typing.Mapping]) -> typing.Tuple[Type, Type]:
-    """return the types of a mapping, e.g. typing.Dict"""
-    return typeinfo.__args__  # type: ignore
-
-
 def check_option_type(name: str, value: typing.Any, typeinfo: Type) -> None:
     """
     Check if the provided value is an instance of typeinfo and raises a
     TypeError otherwise. This function supports only those types required for
     options.
     """
-    e = TypeError("Expected {} for {}, but got {}.".format(
-        typeinfo,
-        name,
-        type(value)
-    ))
+    e = TypeError("Expected {} for {}, but got {}.".format(typeinfo, name, type(value)))
 
-    typename = str(typeinfo)
+    origin = typing.get_origin(typeinfo)
 
-    if typename.startswith("typing.Union") or typename.startswith("typing.Optional"):
-        for T in union_types(typeinfo):
+    if origin is typing.Union or origin is UnionType:
+        for T in typing.get_args(typeinfo):
             try:
                 check_option_type(name, value, T)
             except TypeError:
@@ -48,8 +30,8 @@ def check_option_type(name: str, value: typing.Any, typeinfo: Type) -> None:
             else:
                 return
         raise e
-    elif typename.startswith("typing.Tuple"):
-        types = tuple_types(typeinfo)
+    elif origin is tuple:
+        types = typing.get_args(typeinfo)
         if not isinstance(value, (tuple, list)):
             raise e
         if len(types) != len(value):
@@ -57,18 +39,18 @@ def check_option_type(name: str, value: typing.Any, typeinfo: Type) -> None:
         for i, (x, T) in enumerate(zip(value, types)):
             check_option_type(f"{name}[{i}]", x, T)
         return
-    elif typename.startswith("typing.Sequence"):
-        T = sequence_type(typeinfo)
+    elif origin is abc.Sequence:
+        T = typing.get_args(typeinfo)[0]
         if not isinstance(value, (tuple, list)):
             raise e
         for v in value:
             check_option_type(name, v, T)
-    elif typename.startswith("typing.IO"):
+    elif origin is typing.IO or typeinfo in (typing.TextIO, typing.BinaryIO):
         if hasattr(value, "read"):
             return
         else:
             raise e
-    elif typename.startswith("typing.Any"):
+    elif typeinfo is typing.Any:
         return
     elif not isinstance(value, typeinfo):
         if typeinfo is float and isinstance(value, int):
@@ -77,14 +59,14 @@ def check_option_type(name: str, value: typing.Any, typeinfo: Type) -> None:
 
 
 def typespec_to_str(typespec: typing.Any) -> str:
-    if typespec in (str, int, bool):
+    if typespec in (str, int, float, bool):
         t = typespec.__name__
     elif typespec == typing.Optional[str]:
-        t = 'optional str'
-    elif typespec == typing.Sequence[str]:
-        t = 'sequence of str'
+        t = "optional str"
+    elif typespec in (typing.Sequence[str], abc.Sequence[str]):
+        t = "sequence of str"
     elif typespec == typing.Optional[int]:
-        t = 'optional int'
+        t = "optional int"
     else:
         raise NotImplementedError
     return t
